class Measurement:
    def lightyeardist_to_miles(self, lightyears):
        return lightyears * 5880000000000
    
    def lightyeardist_to_kilometers(self, lightyears):
        return lightyears * 149597870.7
    
    def kilometers_to_lightyeardist(self, kilometers):
        return kilometers / 149597870.7
    
    def miles_to_lightyeardist(self, miles):
        return miles / 5880000000000
   
    def miles_to_au(self, miles):
        return miles / 92955807.267433 
      
    def au_to_miles(self, au):
        return au * 92955807.267433 
      
    def miles_to_parsec(self, miles):
        return miles / self.lightyeardist_to_miles(3.26)
      
    def parsec_to_miles(self, parsecs):
        return self.lightyeardist_to_miles(3.26) * parsecs
    
    def au_to_kilometers(self, au):
        return au * 149597870.7
    
    def kilometers_to_au(self, kilometers, rounded):
        if rounded == True:
            return kilometers // 149597870.7
        else:
            return kilometers / 149597870.7
    
    def kilometers_to_parsec(self, kilometers):
        return kilometers / self.lightyeardist_to_kilometers(3.26)
    
    def parsec_to_kilometers(self, parsecs):
        return self.lightyeardist_to_kilometers(3.26) * parsecs
    
    



class Stellar:
            def getViewableConstellations(self, latitude):
                        constellations = {
                              "aquarius": 0,
                              "aries": 2,
                              "cancer": 2,
                              "capricornus": 1,
                              "gemini": 2,
                              "leo": 0,
                              "libra": 1,
                              "ophiuchus":0,
                              'pisces': 0,
                              'sagittarius':1,
                              'scorpius':1,
                              'taurus': 0,
                              'virgo': 0,
                              'andromeda': 2,
                              'aquila': 0,
                              'ara': 1,
                              "argo navis: Carina Puppis Vela": 1,
                              'auriga': 2,
                              'Boötes': 2,
                              'Canis Major': 1,
                              'Canis Minor': 0,
                              'cassiopeia': 2,
                              'centaurus': 1,
                              'cepheus': 1,
                              'cetus': 0,
                              'coma berenices': 1,
                              'corona australis': 1,
                              'corona borealis': 2,
                              'corvus and crater': 1,
                              'cygnus': 2,
                              'delphinus': 2,
                              'draco': 2,
                              'equuleus': 2,
                              'eridanus': 0,
                              'hercules': 2,
                              'hydra': 0,
                              'lepus': 1,
                              'lupus': 1,
                              'lyra': 2,
                              'orion': 0,
                              'pegasus': 2,
                              'perseus': 2,
                              'piscis austrinus': 1,
                              'sagitta': 2,
                              'serpens': 0,
                              'triangulum': 2,
                              'ursa major': 2,
                              'ursa minor': 2,
                              # animal constellations
                              'apus': 1,
                              'camelopardalis': 2,
                              'canes venatici': 2,
                              'chameleon': 1,
                              'comlumba': 1,
                              'dorado': 1,
                              'grus': 1,
                              'hydrus':1,
                              'indus':1,
                              'lacerta': 2,
                              'leo minor': 2,
                              'lynx': 2,
                              'monoceros': 0,
                              'musca': 1,
                              'pavo': 1,
                              'phoenix': 1,
                              'tucana': 1,
                              'volans': 1,
                              'vulpecula': 2, 
                              # modern constellations
                              'antlia': 1,
                              'caelum': 1,
                              'circinus': 1,
                              'crux': 1,
                              'fornax': 1,
                              'horologium': 1,
                              'mensa': 1,
                              'microscopium':1,
                              'norma': 1,
                              'octans': 1,
                              'pictor': 1,
                              'pyxis': 1,
                              'reticulum': 1,
                              'sculptor': 1,
                              'scutum': 1,
                              'sextans': 0,
                              'telescsopium': 1,
                              'triangulum australe': 1,
                        }
                        hemisphere = 0
                        if latitude > 0:
                            hemisphere = 2
                        else:
                            hemisphere = 1

                        viewable = []
                        for k,v in constellations.items():
                            if v == hemisphere:
                                viewable.append(k)
                            if v == 0:
                                viewable.append(k)


                        return viewable





import math
import datetime


# start by calculating julian date
class Sun:
    def __init__(self):

        self.dayMs = 1000 * 60 * 60 * 24
        self.J1970 = 2440588
        self.J2000 = 2451545

    def toJulian(self, date):
        return date.timestamp() / self.dayMs - 0.5 + self.J1970

    def fromJulian(self, j):
        return datetime.datetime.fromtimestamp((j + 0.5 - self.J1970) * self.dayMs)

    def toDays(self):
        date = datetime.datetime.now()
        return self.toJulian(date) - self.J2000

    def get_julian_date(self):
        date = None
        # If no date is provided, use the current date and time
        if date is None:
            date = datetime.datetime.now()

        # Convert the date to a timestamp in milliseconds
        timestamp = date.timestamp() * 1000

        # Calculate the timezone offset in minutes
        if date.utcoffset():
            timezone_offset = date.utcoffset().total_seconds() // 60
        else:
            timezone_offset = 0

        # Calculate the Julian date and return it
        julian_date = (timestamp / 86400000) - (timezone_offset / 1440) + 2440587.5
        return julian_date

    def solardistance(self):
        juliandate = self.get_julian_date()
        # days since greenwich noon
        n = juliandate - 2451545
        # positions
        meanlong = 280.460 + 0.9856474 * n
        # g is mean anomaly
        g = 357.528 + 0.9856003 * n
        tmp = math.cos(g)
        temptwo = math.cos(2 * g)
        # solar distance is in astronomical units
        solardistance = 1.00014 - 0.01671 * tmp - 0.00014 * temptwo
        return solardistance * 92955807.3  # miles

    def declination(self, l, b):
        e = (math.pi / 180) * 23.4397
        return math.asin(math.sin(b) * math.cos(e) + math.cos(b) * math.sin(e) * math.sin(l))


    def observerAngle(self, height):
        return -2.076 * math.sqrt(height) / 60
    
    
    
    def eclipticlongitude(self, anomaly):
        tmp = math.pi / 180 * (
        1.9148 * math.sin(anomaly) + 0.02 * math.sin(2 * anomaly) + 0.0003 * math.sin(3 * anomaly))
        sectemp = math.pi / 180 * 102.9372
        return tmp + sectemp + math.pi



    def rightAscension(self, l, b):
        e = math.pi / 180 * 23.4397
        return math.atan2(math.sin(l) * math.cos(e) - math.tan(b) * math.sin(e), math.cos(l))

    def azimuth(self, h, phi, declination):
        return math.atan2(math.sin(h), math.cos(h) * math.sin(phi) - math.tan(declination) * math.cos(phi))



    def hourangle(self):
        # it is approximate (doesn't take into account minutes)
        if datetime.datetime.now().hour > 12:
            return 15 * (datetime.datetime.now().hour - 12)
        else:
            return 15 * (-(datetime.datetime.now().hour) + 12)
            
    def mean_solar_time(self, longitude):
        solartime = self.get_julian_date() - longitude/360
        return solartime # returns in approximate mean solar time
   
    def solar_mean_anomaly(self, longitude):
        solartime = self.mean_solar_time(longitude)
        anomaly = (357.5291 + 0.98560028 * solartime)%360
        return anomaly
    
    def center_equation(self, longitude):
        m = self.solar_mean_anomaly(longitude)
        c = 1.9148*math.sin(m) + 0.02 * math.sin(2 * m) + 0.0003*math.sin(3*m)
        return c
    




        
        
